@@ -8,6 +8,7 @@ char *Key=NULL;
 char *LocalHost=NULL;
 ListNode *IncludeExclude=NULL;
 int MatchCount=0, DiffCount=0;
+time_t Now;
 
 const char *HashratHashTypes[]={"md5","sha1","sha256","sha512","whirl","whirlpool","jh-224","jh-256","jh-384","jh-512",NULL};
 
@@ -20,9 +21,9 @@ HashratCtx *Ctx;
 
 Ctx=(HashratCtx *) p_Ctx;
 STREAMClose(Ctx->Out);
-DestroyString(Ctx->HashType);
-DestroyString(Ctx->ListPath);
-DestroyString(Ctx->HashStr);
+Destroy(Ctx->HashType);
+Destroy(Ctx->HashStr);
+Destroy(Ctx->Targets);
 HashDestroy(Ctx->Hash);
 free(Ctx);
 }
@@ -94,8 +95,8 @@ else
 
 STREAMWriteString(Line,Out);
 
-DestroyString(Tempstr);
-DestroyString(Line);
+Destroy(Tempstr);
+Destroy(Line);
 
 return(TRUE);
 }
@@ -123,7 +124,7 @@ char *Tempstr=NULL;
 	}
 
 	if (Ctx->Aux) HashratOutputInfo(Ctx, Ctx->Aux, Path, Stat, Hash);
-	DestroyString(Tempstr);
+	Destroy(Tempstr);
 }
 
 
@@ -137,7 +138,7 @@ STREAM *S;
 		//must quote twice to get through system comamnd
     QuotedPath=QuoteCharsInStr(QuotedPath, Path,"\"'`!|;<> 	");
     QuotedOther=QuoteCharsInStr(QuotedOther, Other,"\"'`!|;<> 	");
-		S=STREAMSpawnCommand("/bin/sh","","",0);
+		S=STREAMSpawnCommand("/bin/sh",0);
 		if (S)
 		{
     	Tempstr=MCopyStr(Tempstr, DiffHook," ",QuotedPath, " ", QuotedOther, ";exit\n",NULL);
@@ -153,8 +154,30 @@ STREAM *S;
 		}
   }
 
-DestroyString(Tempstr);
-DestroyString(QuotedPath);
-DestroyString(QuotedOther);
+Destroy(Tempstr);
+Destroy(QuotedPath);
+Destroy(QuotedOther);
 }
 
+
+char *ReformatHash(char *RetStr, const char *Str, int OutputLen, int SegmentSize, char SegmentChar)
+{
+int ipos=0, opos=0;
+
+if (OutputLen==0) OutputLen=StrLen(Str);
+
+for (ipos=0; ipos < OutputLen; ipos++)
+{
+	//don't increment pos, the loop does this one
+	RetStr=AddCharToBuffer(RetStr, opos++, Str[ipos]);
+	if (Str[ipos]=='\0') break;
+
+	if ((SegmentSize > 0) && (ipos > 0) && (OutputLen-ipos > 1) && (((ipos+1) % SegmentSize)==0)) 
+	{
+		RetStr=AddCharToBuffer(RetStr, opos++, SegmentChar);
+	}
+
+}
+
+return(RetStr);
+}
