@@ -2,7 +2,7 @@
 #ifndef HASHRAT_COMMON_H
 #define HASHRAT_COMMON_H
 
-#include "libUseful-2.5/libUseful.h"
+#include "libUseful-3/libUseful.h"
 #include "glob.h"
 
 
@@ -26,16 +26,20 @@
 #define ACT_CHECKBACKUP 25
 
 
+#define FLAG_NEXTARG 1
 //Two flags with the same values, but used in different contexts
 #define FLAG_ERROR 2
 #define FLAG_VERBOSE 2
-#define FLAG_DEVMODE 8
-#define FLAG_DIR_INFO 32
-#define FLAG_TRAD_OUTPUT 64
-#define FLAG_BSD_OUTPUT  128 
-#define FLAG_XSELECT     256 
-#define FLAG_MEMCACHED  1024
-#define FLAG_OUTPUT_FAILS 2048 
+//---------------------------
+#define FLAG_DEVMODE         8
+#define FLAG_DIR_INFO       32
+#define FLAG_TRAD_OUTPUT    64
+#define FLAG_BSD_OUTPUT    128 
+#define FLAG_XSELECT       256 
+#define FLAG_XATTR         512
+#define FLAG_TXATTR       1024
+#define FLAG_MEMCACHED    2048
+#define FLAG_OUTPUT_FAILS 8192 
 #define FLAG_FULLCHECK   32768
 #define FLAG_HMAC        65536
 #define FLAG_HIDE_INPUT 131072
@@ -46,18 +50,33 @@
 #define FLAG_NET       8388608
 #define FLAG_UPDATE   16777216
 
-#define CTX_CACHED   1
-#define CTX_RECURSE  2
-#define CTX_ONE_FS  16
-#define CTX_DEREFERENCE 128
-#define CTX_STORE_XATTR 256
-#define CTX_STORE_MEMCACHED 512
-#define CTX_STORE_FILE 1024
-#define CTX_XATTR_ROOT 2048
-#define CTX_XATTR_CACHE 4096
-#define CTX_EXES 8192
-#define CTX_INCLUDE 16384
-#define CTX_EXCLUDE 32768
+//map cgi options to console-mode options that make no sense in .cgi mode
+#define CGI_DOHASH FLAG_NEXTARG
+#define CGI_NOOPTIONS FLAG_TRAD_OUTPUT
+#define CGI_HIDETEXT  FLAG_BSD_OUTPUT
+#define CGI_SHOWTEXT  FLAG_XSELECT
+
+//the first few of these must be low because they will go in a 16-bit value in
+//the 'include/exclude' list
+#define CTX_INCLUDE 2
+#define CTX_EXCLUDE 4
+#define CTX_MTIME   8
+#define CTX_MMIN    16
+#define CTX_MYEAR    32
+#define CTX_RECURSE  128
+#define CTX_HASH_AND_RECURSE  256
+#define CTX_DEREFERENCE 512
+#define CTX_XATTR 1024
+#define CTX_MEMCACHED 2048
+#define CTX_STORE_XATTR 4096
+#define CTX_STORE_MEMCACHED 8192
+#define CTX_STORE_FILE  16384
+#define CTX_XATTR_ROOT  32768
+#define CTX_XATTR_CACHE 65536
+#define CTX_EXES  131072          
+#define CTX_CACHED 262144
+#define CTX_ONE_FS 524288
+
 
 #define RESULT_PASS 1
 #define RESULT_FAIL 2
@@ -66,13 +85,15 @@
 #define FLAG_RESULT_MASK 15
 #define RESULT_RUNHOOK 8192
 
+
+
 #define FP_HASSTAT 1
 
 #define BLOCKSIZE 4096
 
 #define IGNORE -1
 
-#define VERSION "1.8.12"
+#define VERSION "1.10.2"
 
 
 typedef struct
@@ -92,13 +113,14 @@ typedef struct
 STREAM *NetCon;
 STREAM *Out; 
 STREAM *Aux; 
-THash *Hash;
+HASH *Hash;
 char *HashType;
-char *ListPath;
 char *HashStr;
+char *Targets;
 int Action;
 int Flags;
-int Encoding;
+int Encoding;  
+dev_t StartingFS;
 ListNode *Vars;
 } HashratCtx;
 
@@ -110,11 +132,13 @@ extern char *LocalHost;
 extern const char *HashratHashTypes[];
 extern ListNode *IncludeExclude;
 extern int MatchCount, DiffCount;
+extern time_t Now;
 
 TFingerprint *TFingerprintCreate(const char *Hash, const char *HashType, const char *Data, const char *Path);
 void HashratCtxDestroy(void *p_Ctx);
 void HashratStoreHash(HashratCtx *Ctx, const char *Path, struct stat *Stat, const char *Hash);
 int HashratOutputInfo(HashratCtx *Ctx, STREAM *S, const char *Path, struct stat *Stat, const char *Hash);
 void RunHookScript(const char *Hook, const char *Path, const char *Other);
+char *ReformatHash(char *RetStr, const char *Str, int OutputLen, int SegmentSize, char SegmentChar);
 
 #endif
